#include "argparse.h"
#include "logger.h"
#include "parser/ast.h"
#include "parser/parser.h"
#include "dynarray.h"
#include "prettyprint.h"
#include "token.h"
#include <util.h>
#include <tokeniser.h>
#include <stdio.h>
#include <compiler.h>

int main(int argc, const char** argv) {
    args_t* args = parse_args(argc, argv);
    if (!args) {
        return 1;
    }

    char* str = read_to_string(args->input_files[0]);
    if (!str) return 1;
    token_t* tokens = tokenise_string((char*)args->input_files[0], str);
    if (!tokens) return 1;
    int token_count = dynarray_length(tokens);

    for (int i = 0; i < token_count; i++) {
        log_debug(&tokens[i].loc, "Token: %s", get_tok_str_dbg(&tokens[i]));
    }
    program_t* prog = parse(tokens);
    if (!prog) return 1;
    pretty_print(tokens);
    compile(args, prog);
    return 0;
}
