#include "logger.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <token.h>
#include <dynarray.h>
#include <parser/parser.h>
#include <parser/precomp.h>
#include <parser/tokcmp.h>
#include <parser/ast.h>

token_t* expect_token_type(parser_state_t* state, token_type_t type) {
    const char* exoected_type_str = get_tok_type_str_dbg(type);
    da_debug_t* dbg = (da_debug_t*)((size_t*)state->tokens-3); 
    if (dynarray_pop(state->tokens, &state->curr_tok) != 0) {
        log_error(&state->curr_tok.loc, "Invalid word, expected %s, got nothing.", exoected_type_str);
        return NULL;
    }

    const char* s1 = get_tok_str_dbg(&state->curr_tok);
    // log_debug(&state->curr_tok.loc, "Found %s", s1);
    if (state->curr_tok.type != type) {
        log_error(&state->curr_tok.loc, "Invalid word, expected %s, got %s.", exoected_type_str, s1);
        return NULL;
    }
    return &state->curr_tok;
}

token_t* expect_token_type_mul(parser_state_t* state, token_type_t types[], int type_count) {

    char* expected_type_str = malloc(1024 * 4);
    int offset = 0;
    for (int i = 0; i < type_count; i++) {
        offset += snprintf(expected_type_str + offset, 1024, " %s", get_tok_type_str_dbg(types[i]));
    }

    if (dynarray_pop(state->tokens, &state->curr_tok) != 0) {
        log_error(&state->curr_tok.loc, "Invalid word, expected %s, got nothing.", expected_type_str);
        return NULL;
    }

    for (int i = 0; i < type_count; i++) {
        if (state->curr_tok.type == types[i]) {
            return &state->curr_tok;
        }
    }
    const char* s1 = get_tok_str_dbg(&state->curr_tok);
    log_error(&state->curr_tok.loc, "Invalid word, expected %s, got %s.", expected_type_str, s1);
    return NULL;
}

token_t* expect_token_type_ex(parser_state_t* state, token_type_t type, int inner_type) {
    const char* exoected_type_str = get_tok_type_str_dbg(state->curr_tok.type);
    if (dynarray_pop(state->tokens, &state->curr_tok) != 0) {
        log_error(&state->curr_tok.loc, "Invalid word, expected %s, got nothing.", exoected_type_str);
        return NULL;
    }
    // NOTE: all inner types are in a union
    if (state->curr_tok.type != type || state->curr_tok.kw_type != inner_type) {
        const char* s1 = get_tok_str_dbg(&state->curr_tok);
        log_error(&state->curr_tok.loc, "Invalid word, expected %s, got %s.", exoected_type_str, s1);
        return NULL;
    }
    return &state->curr_tok;
}

token_t* test_token_type_ex(parser_state_t* state, token_type_t tok_type, int gen_type) {
    const char* exoected_type_str = get_tok_type_str_dbg(state->curr_tok.type);
    token_t* token = &state->tokens[dynarray_length(state->tokens) - 1];
    // NOTE: tests inner type in a generic way
    if (token->type != tok_type || ((int)token->kw_type) != gen_type) {
        return NULL;
    }
    return token;
}

token_t* test_token_type(parser_state_t* state, token_type_t tok_type) {
    const char* exoected_type_str = get_tok_type_str_dbg(state->curr_tok.type);
    token_t* token = &state->tokens[dynarray_length(state->tokens) - 1];
    if (token->type != tok_type) {
        return NULL;
    }
    return token;
}
